using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.OperationModel;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;

namespace TestOPC
{
    class Program
    {
        static readonly object locker = new object();

        static void Main(string[] args)
        {
            EasyUAClientConfiguration config = new EasyUAClientConfiguration();
            config.SharedParameters.EngineParameters.CertificateAcceptancePolicy.AcceptAnyCertificate = true;
            EasyUAClient.LogEntry += EasyUAClient_LogEntry;

            var sw = new Stopwatch();
            var tasks = new List<Task>();

            Ips.ToList().ForEach((ip) => tasks.Add(new Task(() => ReadMultiple(new Connection(ip)))));

            Console.WriteLine($"Starting {tasks.Count()} tasks ...");
            sw.Start();
            tasks.ToList().ForEach(t => t.Start());
            Task.WaitAll(tasks.ToArray());
            sw.Stop();
            Console.WriteLine($"Elapsed {sw.ElapsedMilliseconds}ms.");

            Console.ReadKey();
        }

        private static void EasyUAClient_LogEntry(object sender, OpcLabs.BaseLib.Instrumentation.LogEntryEventArgs e)
        {
            if (e.EventId == 3201)
                Console.WriteLine($"{e.EventId}: {e.Message}");
        }


        static void ReadMultiple(Connection conn)
        {
            UAAttributeDataResult[] values;
            var args = new List<UAReadArguments>();

            foreach (var n in nodeNames)
            {
                var endPoint = $"opc.tcp://{conn.Ip}:4840";
                var node = $"ns=3;s=\"OPC_Datenbaustein\".\"{n}\"";
                args.Add(new UAReadArguments(endPoint, node));
            }
            //lock (locker)   // Use these lock or not ...
            {
                values = conn.EasyUAClient.ReadMultiple(args.ToArray());
            }
            Console.WriteLine($"READ from {conn.Ip} {values.Count()} values, first has {values[0].StatusInfo}.");
        }

        public class Connection
        {
            public Connection(string ip)
            {
                Ip = ip;
                EasyUAClient = new EasyUAClient();
            }
            public string Ip;
            public EasyUAClient EasyUAClient;
        }

        static readonly string[] nodeNames = new string[] {
            "plcPlaceFree",
            "plcPlaceError",
            "plcContainerID",
            "plcContainerIdentity",
            "plcContainerLoad",
            "plcContainerCleanState",
            "plcReservationResponse",
            "plcContainerDataReady",
            "plcContainerDataAck",
            "agvReservationRequest",
            "agvContainerDataReady",
            "agvContainerDataAck",
            "agvContainerID",
            "agvContainerIdentity",
            "agvContainerLoad",
            "agvContainerCleanState",
            "agvContainerTarget",
            "agvEntryRequest",
            "agvInsideArea",
        };

        static readonly string[] Ips = new string[] {
            "10.199.12.51",
            "10.199.12.52",
            "10.199.12.53",
            "10.199.12.60",
            "10.199.12.61",
            "10.199.12.62",
            "10.199.12.63",
            "10.199.12.64",
            "10.199.12.65",
            "10.199.12.66",
            "10.199.12.70",
            "10.199.12.71",
            "10.199.12.72",
            "10.199.12.73",
            "10.199.12.74",
            "10.199.12.75",
            "10.199.12.76",
            "10.199.12.77",
            "10.199.12.78",
            "10.199.12.79",
            "10.199.12.80",
            "10.199.12.81",
            "10.199.12.82",
            "10.199.12.83",
            "10.199.12.84",
            "10.199.12.85",
            "10.199.12.86",
            "10.199.12.87",
            "10.199.12.88",
            "10.199.12.89",
        };
    }
}